pub struct SegmentedPriorityQueue<T> { /* private fields */ }Expand description
Like a [PriorityQueue] but avoids starvation by
using a rotating bip-buffer of priority queues.
Implementations
Sourceimpl<T> SegmentedPriorityQueue<T>
impl<T> SegmentedPriorityQueue<T>
pub fn new() -> SegmentedPriorityQueue<T>
Sourcepub fn push(&self, t: T, priority: u64)
pub fn push(&self, t: T, priority: u64)
Higher priority tends to gets popped first.
The internal bip buffer of priority queues gets rotated when the read side is empty.
Examples
let pq = komora_sync::SegmentedPriorityQueue::new();
pq.push(2, 2);
pq.push(1, 1);
// internal state:
// read buffer: []
// write buffer: [2, 1]
// queue rotated on pop when read buffer is empty
assert_eq!(pq.pop(), 2);
// internal state:
// read buffer: [1]
// write buffer: []
// new writes get pushed to write buffer
pq.push(4, 4);
pq.push(3, 3);
// internal state:
// read buffer: [1]
// write buffer: [4, 3]
// this is the last element in the reader side
assert_eq!(pq.pop(), 1);
// queue rotated on pop when reader is empty
assert_eq!(pq.pop(), 4);
assert_eq!(pq.pop(), 3);pub fn pop(&self) -> T